#include        "EXTERN.h"
#include        "perl.h"
#include        "XSUB.h"

#include        "isc-dhcp/result.h"
#include        "omapip/omapip.h"
#include	"dhcpctl.h"

#include 	<stdio.h>
#include	<netinet/in.h>
#include        <sys/socket.h>
#include        <sys/time.h>


MODULE  = OMAPI::CORE		PACKAGE = OMAPI::CORE
PROTOTYPES: DISABLE

dhcpctl_handle *
Connect (address, port, key_name, key_type, key)
    char *		address
    int 		port
    char *		key_name
    char *		key_type
    char *		key

    PREINIT:
	dhcpctl_handle *	secret = NULL;
	dhcpctl_handle *	handle = NULL;

	isc_result_t		status;
	
    CODE:
	RETVAL = (dhcpctl_handle *) malloc (sizeof (dhcpctl_handle));
	*RETVAL = dhcpctl_null_handle;
	dhcpctl_initialize ();

	fprintf (stderr, "Connecting to %s : %d\n", address, port);

	if (key_name != NULL) {
	    secret = (dhcpctl_handle *) malloc (sizeof (dhcpctl_handle));
	    *secret = dhcpctl_null_handle;

	    status = dhcpctl_new_authenticator (secret, key_name, key_type,
		key, strlen(key) + 1);

	    if (status != ISC_R_SUCCESS) {
	        fprintf (stderr, "authenticator failed (%d)\n", status);
	        free (RETVAL);
	        free (RETVAL);
	        RETVAL = NULL;
	    }
	}

	if (RETVAL != NULL) {
	    status = dhcpctl_connect (RETVAL, address, port, *secret);

	    if (status != ISC_R_SUCCESS) {
	        fprintf (stderr, "connect failed (%d)\n", status);
	        free (RETVAL);
	        RETVAL = NULL;
	    }
	}

    OUTPUT: RETVAL

dhcpctl_handle *
New_Object (server, type, flag, data)
    dhcpctl_handle *		server
    char *			type
    char *			flag
    SV *			data

    PREINIT:
	HE *			entry;
	HV *			hash;
	SV *			edata;
	char *			ename;
        I32			ksize;
	char *			epntr;
	STRLEN			esize;
	int			cflag;

        dhcpctl_data_string	mdata;
	isc_result_t		status;
	isc_result_t		waitrc;

    INIT:
	if ((! SvROK (data)) || (SvTYPE (SvRV (data)) != SVt_PVHV)) {
	    fprintf (stderr, "object data is not a hash reference.\n");
	    XSRETURN_UNDEF;
	}

    CODE:
        RETVAL = (dhcpctl_handle *) malloc (sizeof (dhcpctl_handle));
        *RETVAL = dhcpctl_null_handle;

        status = dhcpctl_new_object (RETVAL, *server, type);
        if (status != ISC_R_SUCCESS) {
            fprintf (stderr, "Unable to create object (%d)\n", status);
            free (RETVAL);
            RETVAL = NULL;
        }

	hash = (HV *) SvRV (data);
	hv_iterinit (hash);

        while (edata = hv_iternextsv (hash, &ename, &ksize)) {
            epntr = SvPV  (edata, esize); 

            fprintf (stderr, "Processing data entry %s (%d)\n", ename, esize);
            fprintf (stderr, "data pointer = %8.8X\n", epntr);

	    memset (&mdata, 0, sizeof (mdata));
	    status = omapi_data_string_new (&mdata, esize, MDL);
            memcpy (mdata->value, epntr, esize);

            status = dhcpctl_set_value (*RETVAL, mdata, ename);
            if (status != ISC_R_SUCCESS) {
                fprintf (stderr, "Unable to set select value (%d)\n", status);
                free (RETVAL);
                RETVAL = NULL;
	    }

	    dhcpctl_data_string_dereference (&mdata, MDL);
        }

	if (strncasecmp (flag, "CREATE", 7) == 0) {
	    cflag = DHCPCTL_CREATE | DHCPCTL_EXCL;
        } else {
	    cflag = 0;
        }

        status = dhcpctl_open_object (*RETVAL, *server, cflag);
        if (status != ISC_R_SUCCESS) {
            fprintf (stderr, "Unable to open object (%d)\n", status);
            free (RETVAL);
            RETVAL = NULL;
        }

        status = dhcpctl_wait_for_completion (*RETVAL, &waitrc);
        if (status == ISC_R_SUCCESS) {
            if ((waitrc != ISC_R_SUCCESS) && (waitrc != ISC_R_NOTFOUND)) {
                fprintf (stderr, "wait returned an error (%d)\n", waitrc);
                free (RETVAL);
                RETVAL = NULL;
            } else if (waitrc == ISC_R_NOTFOUND) {
                fprintf (stderr, "object not found\n");
                free (RETVAL);
                RETVAL = NULL;
            }
        } else {
            fprintf (stderr, "Unable to issue a wait (%d)\n", status);
            free (RETVAL);
            RETVAL = NULL;
        }

    OUTPUT: RETVAL

char *
Get_Value (object, variable, type)
    dhcpctl_handle *		object
    char *			variable
    char *			type

    PREINIT:
	isc_result_t		status;
	dhcpctl_data_string	value			= NULL;
	unsigned int		temp;

    CODE:
	status = dhcpctl_get_value (&value, *object, variable);
	ST(0) = sv_newmortal();

	if (status != ISC_R_SUCCESS) {
	    if (status != ISC_R_NOTFOUND) {
	        fprintf (stderr, "Unable to get value (%d)\n", status);
	    }
	} else {
	    if ((*type == 'H') || (*type == 'I') || (*type == 'T')) {
	        memcpy (&temp, value->value, value->len);
	        sv_setiv (ST(0), temp);
	    }

	    if ((*type == 'D') || (*type == 'S')) {
	        sv_setpvn (ST(0), value->value, value->len);
	    }

	    dhcpctl_data_string_dereference (&value, MDL);
	}

char *
Set_Value (object, variable, type, value)
    dhcpctl_handle *            object
    char *                      variable
    char *                      type
    char *			value

    PREINIT:
	isc_result_t            status;
	dhcpctl_data_string     xvalue			= NULL;
	unsigned int            temp;
	void *			s_data;
	int			s_size;

    CODE:
	if ((*type == 'H') || (*type == 'I') || (*type == 'T')) {
            temp = (int) SvIV(ST(4));
	    s_data = (void *) &temp;
	    s_size = 4;
	}

	if ((*type == 'D') || (*type == 'S')) {
	    s_data = (void *) value;
	    s_size = strlen (value);
	}

	status = omapi_data_string_new (&xvalue, s_size, MDL);
	memcpy (xvalue->value, s_data, s_size);

	status = dhcpctl_set_value (*object, xvalue, variable);
	if (status  == ISC_R_SUCCESS) {
	    RETVAL = value;
	} else {
            fprintf (stderr, "Unable to set value (%d)\n", status);
	    RETVAL = NULL;
	}

    OUTPUT: RETVAL
