use OMAPI::CORE;
use OMAPI::HREC;
use OMAPI::LREC;

use OMAPI::Host;
use OMAPI::Lease;

package OMAPI::DHCP;
    use strict;
    1;


#=============================================================================
#
sub new {
    my ($package, $server, $port, $secret) = @_;
    my ($key_name, $secret_key);

    $server ||= "127.0.0.1";
    $port   ||= 8001;


    if ($secret =~ m/:/) {
        ($key_name, $secret_key) = split (/:/, $secret);
    } else {
        $secret_key = $secret;
        $key_name = 'OMAPI1';    
    }

    my $self = OMAPI::CORE::Connect ($server, $port, $key_name,
		'hmac-md5', $secret_key);

    return bless ($self, $package); 
}


#=============================================================================
#
sub Select_Lease {
    my ($self, $data1) = @_;

    my $data2 = __Encode OMAPI::Lease ($data1);
    my $handle = OMAPI::CORE::New_Object ($self, 'lease', 'SELECT', $data2);
    return undef unless (defined $handle);

    my $lease = bless ({}, 'OMAPI::Lease');
    unless (tie %$lease, 'OMAPI::LREC', $handle) {
        return undef;
    }

    return $lease;
}


#=============================================================================
#
sub Create_Host {
    my ($self, $data1) = @_;

    my $data2 = __Encode OMAPI::Host ($data1);
    my $handle = OMAPI::CORE::New_Object ($self, 'host', 'CREATE', $data2);
    return undef unless (defined $handle);

    my $host = bless ({}, 'OMAPI::Host');
    unless (tie %$host, 'OMAPI::HREC', $handle) {
        return undef;
    }

    return $host;
}


#=============================================================================
#
sub Select_Host {
    my ($self, $data1) = @_;

    my $data2 = __Encode OMAPI::Host ($data1);
    my $handle = OMAPI::CORE::New_Object ($self, 'host', 'SELECT', $data2);
    return undef unless (defined $handle);

    my $host = bless ({}, 'OMAPI::Host');
    unless (tie %$host, 'OMAPI::HREC', $handle) {
        return undef;
    }

    return $host;
}

__END__

This module provides interfaces to the core OMAPI functionality for use when
manipulating DHCP leases and host declarations.

=head1 NAME

OMAPI::DHCP - Interface for DHCP server management using OMAPI

=head1 SYNOPSIS
  
  use OMAPI::DHCP
  
=head1 DESCRIPTION

C<OMAPI::DHCP> provides interfaces to the core OMAPI functionality 
for use when manipulating DHCP leases and host declarations.

=head1 CONSTRUCTOR

=over 4

=item new($ServerIP, $Port, $Secret)

Creates a new C<OMAPI::DHCP>. All three parameters must be specified.
The Server IP must be the IP address of your server. The Port should be
the OMAPI port to use (7911 is standard). The Secret must be in the format
"keyname:key". Keyname should be the name of the key as specified on the
DHCP server. The key must be the shared key as configured on the server.

The key must NOT be base64 encoded as specified on the server. An easy
way to convert is to use the C<MIME::Base64> module, using the 
C<decode_base64> function.

=head1 SEE ALSO

C<OMAPI::CORE> defines the core OMAPI functions.

=head1 AUTHOR

Bruce Hudson

